<template>
    <section class="games">
        <GameSwitch page="mines" />
        <div class="games__row">
            <div class="mines container row">
                <div class="spinner-border" v-if="isLoading">
                    <span class="sr-only">Loading...</span>
                </div>
                <div class="col-md-5" v-show="!isLoading">
                    <div class="input-title">
                        Количество бомб ({{this.bombs}}):
                    </div>
                    <range-slider
                        class="slider"
                        min="2"
                        max="24"
                        step="1"
                        v-model="bombs"
			:disabled="status_disabled"
                    >
                    </range-slider>
                    <div class="inputs-wrap">
                        <div class="inputs">
                            <div class="input">
                                <div class="input-title">
                                    Сумма ставки:
                                </div>
                                <div class="button_container">
                                    <button 
                                        class="btn-outline__body left"
					:disabled="status_disabled"
                                        @click="amount = 1; validateBet()"
					
                                    >
                                        Min
                                    </button>
                                    <input 
                                        type="number" 
                                        placeholder="Сумма ставки" 
                                        class="form-control" 
                                        v-model="amount"
                                        @change="validateBet"
					:disabled="status_disabled"
                                    />
                                    <button 
                                        class="btn-outline__body right"
					:disabled="status_disabled"
                                        @click="amount = $root.user !== null ? $root.user.balance : 0; validateBet()"
                                    >
                                        Max
                                    </button>
                                </div>
                            </div>
                        </div>
                        <div class="mines__control" v-if="!game.isStart">
                            <button class="btn__create btn-primary__body" @click="startGame" :disabled="createBtn.isLoading">
                                <div class="spinner-border buttonLoader" v-if="createBtn.isLoading">
                                    <span class="sr-only">Loading...</span>
                                </div>
                                <span v-else>Создать игру</span>
                            </button>
                        </div>
                        <div class="mines__control" v-else>
                            <button class="btn__create btn-primary__body w-48" @click="catchMines" :disabled="takeBtn.isLoading">
                                <div class="spinner-border buttonLoader" v-if="takeBtn.isLoading">
                                    <span class="sr-only">Loading...</span>
                                </div>
                                <span v-else>Забрать {{ game.win.toFixed(2) }}</span>
                            </button>
                            <button class="btn__create btn-outline__body w-48" @click="autoSelect" :disabled="autoBtn.isLoading">
                                <div class="spinner-border buttonLoader wh-14" v-if="autoBtn.isLoading">
                                    <span class="sr-only">Loading...</span>
                                </div>
                                <span v-else>Автовыбор</span>
                            </button>
                        </div>
                    </div>
                </div>
                <div class="col-md-7" :class="[isLoading ? '' : 'grid']" v-show="!isLoading">
                    <div 
                        :class="[
                            'mines__field',
                            'grid',
                            { 'field__disabled': !game.isStart || loadBomb !== -1}
                        ]"
                    >
                        <div 
                            v-for="(item, index) in grid"
                            :key="index"
                            @click="openPath(index + 1)"
                            :class="[
                                'mine-tile',
                                { 'correct': item.diamond },
                                { 'wrong': item.bomb },
                                { 'active': opened.indexOf(index + 1) !== -1 },
                                { 'loading': loadBomb == item.index }
                            ]"
                        >
                            <div class="spinner-border buttonLoader inCell" v-if="loadBomb == item.index">
                                <span class="sr-only">Loading...</span>
                            </div>
                        </div>
                    </div>
                    <div class="multiplayers__wrap">
                        <div 
                            class="multiplier-item" 
                            v-for="(item, index) in coefs[bombs - 1]"
                        >
                            <div 
                                :class="[
                                    'multiplier',
                                    { 'inProcess': opened.length == index && game.isStart },
                                    { 'active': opened.length > index && game.isStart }
                                ]"
                            >
                                x{{item}}
                            </div>
                            <div class="pointer">→</div>
                        </div>
                    </div>
                </div>
            </div>
            <History />
        </div>
    </section>
</template>

<script>
import GameSwitch from '../components/GameSwitch'
import History from '../components/History'
import { getEmptyArr } from './utils/getEmptyArr'

import RangeSlider from 'vue-range-slider'
import 'vue-range-slider/dist/vue-range-slider.css'

export default {
    components: {
        GameSwitch,
        History,
        RangeSlider
    },
    data() {
        return {
            isLoading: true,
            grid: [],
            opened: [],
            game: {
                isStart: false,
                win: 0
            },
            createBtn: {
                isLoading: false
            },
            takeBtn: {
                isLoading: false
            },
            autoBtn: {
                isLoading: false
            },
            loadBomb: -1,
            amount: 1,
            bombs: 3,
            coefs: [
                [],
                [1.09,1.19,1.3,1.43,1.58,1.75,1.96,2.21,2.5,2.86,3.3,3.85,4.55,5.45,6.67,8.33,10.71,14.29,20,30,50,100,300],
                [1.14,1.3,1.49,1.73,2.02,2.37,2.82,3.38,4.11,5.05,6.32,8.04,10.45,13.94,19.17,27.38,41.07,65.71,115,230,575,2300],
                [1.19,1.43,1.73,2.11,2.61,3.26,4.13,5.32,6.95,9.27,12.64,17.69,25.56,38.33,60.24,100.4,180.71,361.43,843.33,2530,12650],
                [1.25,1.58,2.02,2.61,3.43,4.57,6.2,8.59,12.16,17.69,26.54,41.28,67.08,115,210.83,421.67,948.75,2530,8855,53130],
                [1.32,1.75,2.37,3.26,4.57,6.53,9.54,14.31,22.12,35.38,58.97,103.21,191.67,383.33,843.33,2108.33],
                [1.39,1.96,2.82,4.13,6.2,9.54,15.1,24.72,42.02,74.7,140.06,280.13,606.94,1456.67,4005.83,13352.78],
                [1.47,2.21,3.38,5.32,8.59,14.31,24.72,44.49,84.04,168.08,360.16,840.38,2185,6555,24035,120175,1081575],
                [1.56,2.5,4.11,6.95,12.16,22.12,42.02,84.04,178.58,408.19,1020.47,2857.31,9286.25,37145,204297.5,2042975],
                [1.67,2.86,5.05,9.27,17.69,35.38,74.7,168.08,408.19,1088.5,3265.49,11429.23,49526.67,297160,3268760],
                [1.79,3.3,6.32,12.64,26.54,58.97,140.06,360.16,1020.47,3265.49,12245.6,57146.15,371450,4457400],
                [1.92,3.85,8.04,17.69,41.28,103.21,280.13,840.38,2857.31,11429.23,57146.15,400023.08,5200300],
                [2.08,4.55,10.45,25.56,67.08,191.67,606.94,2185,9286.25,49526.67,371450,5200300],
                [2.27,5.45,13.94,38.33,115,383.33,1456.67,6555,37145,297160,4457400],
                [2.5,6.67,19.17,60.24,210.83,843.33,4005.83,24035,204297.5,3268760],
                [2.78,8.33,27.38,100.4,421.67,2108.33,13352.78,120175,2042975],
                [3.13,10.71,41.07,180.71,948.75,6325,60087.5,1081575],
                [3.57,14.29,65.71,361.43,2530,25300,480700],
                [4.17,20,115,843.33,8855,177100],
                [5,30,230,2530,53130],
                [6.25,50,575,12650],
                [8.33,100,2300],
                [12.5,300],
                [25]
            ],
        }
    },
    mounted() {
        this.grid = getEmptyArr(25, { bombs: false, diamond: false })
        this.init();
        this.validateBet();
    },
    methods: {
        init() {
            this.$root.axios.post('/mines/init')
            .then(response => {
                const {data} = response;
                this.isLoading = false;

                if(typeof data.click !== 'undefined') {
                    this.game.isStart = true;
                    this.amount = data.amount;
                    this.bombs = data.bombs;
                    this.opened = data.click;
                    this.game.win = data.total;

                    this.grid = this.grid.map((item, index) =>
                        data.click.indexOf(index + 1) !== - 1
                            ? { ...item, diamond: true }
                            : item
                    )

                    setTimeout(() => {
                        var multiplierWrap = document.querySelector('.multiplayers__wrap');
                        var multiplierItem = document.querySelector('.multiplier-item');

                        if(this.opened.length % 2 == 0 || this.opened.length % 4 != 0) {
                            multiplierWrap.scrollLeft = multiplierItem.clientWidth * (this.opened.length - 2)
                        }
                    }, 100)

                    this.validateBet();
                }
            })
            .catch(error => {
                if(error.response.status == 401) {
                    this.isLoading = false;
                    return;
                }
                notify({
                    theme: 'red',
                    text: error.response.status
                })
            })
        },
        startGame() {
            var multiplierWrap = document.querySelector('.multiplayers__wrap');

            multiplierWrap.scrollTo({
                left: 0,
                behavior: 'smooth'
            });
            this.createBtn.isLoading = true;
	    this.status_disabled = true;
            this.grid = getEmptyArr(25, { bombs: false, diamond: false })
            this.opened = []
            this.game.win = 0;

            this.$root.axios.post('/mines/start', {
                amount: this.amount,
                bombs: this.bombs
            })
            .then(response => {
                const {data} = response;
                this.createBtn.isLoading = false;

                if(data.error) {
		    this.status_disabled = false;
                    return notify({
                        theme: 'red',
                        text: data.message
                    })
                }

                this.$root.user.balance = data.balance;
                this.game.isStart = true;
                this.game.win = 0;
            })
            .catch(error => {
                this.createBtn.isLoading = false;
		this.status_disabled = false;
                notify({
                    theme: 'red',
                    text: error.response.status
                })
            })
        },
        openPath(index) {
            var multiplierWrap = document.querySelector('.multiplayers__wrap');
            var multiplierItem = document.querySelector('.multiplier-item');

            this.loadBomb = index - 1;
            this.$root.axios.post('/mines/open', {
                path: index
            })
            .then(response => {
                const {data} = response;
                this.autoBtn.isLoading = false;
                this.loadBomb = -1;
                
                if(data.error) {
		    this.status_disabled = false;
                    return notify({
                        theme: 'red',
                        text: data.message
                    })
                }
                
                this.opened.push(index);

                if(data.continue) { // win - continue
                    this.game.win = data.total;
                    this.grid = this.grid.map(item =>
                        item.index === index - 1
                            ? { ...item, diamond: true }
                            : item
                    )
                } else { // lose - stop
                    multiplierWrap.scrollTo({
                        left: 0,
                        behavior: 'smooth'
                    });
		    this.status_disabled = false;
                    this.game.isStart = false;
                    this.grid = this.grid.map((item, index) =>
                        data.bombs.indexOf(index + 1) !== - 1
                            ? { ...item, bomb: true }
                            : { ...item, diamond: true }
                    )
                }

                if((this.opened.length % 3 == 0 || this.opened.length >= 3) && data.continue) {
                    multiplierWrap.scrollTo({
                        left: multiplierItem.clientWidth * (this.opened.length - 2),
                        behavior: 'smooth'
                    })
                }
            })
            .catch(error => {
                this.autoBtn.isLoading = false;
		this.status_disabled = false;
                this.loadBomb = -1;
                notify({
                    theme: 'red',
                    text: error.response.status
                })
            })
        },
        catchMines() {
            this.takeBtn.isLoading = true;
            this.$root.axios.post('/mines/take')
            .then(response => {
                const {data} = response;
                this.takeBtn.isLoading = false;
                if(data.error) {
		    this.status_disabled = true;
                    return notify({
                        theme: 'red',
                        text: data.message
                    })
                }
                
                var multiplierWrap = document.querySelector('.multiplayers__wrap');
                multiplierWrap.scrollTo({
                    left: 0,
                    behavior: 'smooth'
                });

                this.$root.user.balance = data.balance;
                this.game.isStart = false;
		this.status_disabled = false;
                this.grid = getEmptyArr(25, { bombs: false, diamond: false });
                this.opened = [];
                notify({
                    theme: 'green',
                    text: data.message
                })
            })
            .catch(error => {
                this.takeBtn.isLoading = false;
		this.status_disabled = true;
                notify({
                    theme: 'red',
                    text: error.response.status
                })
            })
        },
        autoSelect() {
            var noActive = document.querySelectorAll('.mine-tile:not(.active)')
            noActive[Math.floor(Math.random() * noActive.length)].click();
            this.autoBtn.isLoading = true;
        },
        validateBet() {
            this.amount = parseFloat(this.amount).toFixed(2)
        }
    },
}
</script>

<style scoped>
.button_container {
    display: flex;
    min-width: 100%;
    justify-content: space-between;
}

.button_container .btn-outline__body.left {
    border-right-color: none!important;
    border-radius: 4px 0 0 4px!important;
}

.button_container .btn-outline__body.right {
    border-left-color: none!important;
    border-radius: 0 4px 4px 0!important;
}

.inCell {
    color: #d5d4d4;
    font-size: 10px;
}

.buttonLoader {
    width: 16px!important;
    height: 16px!important;
}

.field__disabled {
    cursor: not-allowed;
}

.mines__field.field__disabled > .mine-tile {
    pointer-events: none
}

.slider {
    min-width: 100%
}

.range-slider-fill {
    background-color: #2c80ff!important;
}

.mines {
    padding: 32px 0;
    background: #fff;
    border-radius: 0 0 8px 8px;
}

.grid {
    display: grid!important;
}

.mines__field {
    grid-template-columns: repeat(5, minmax(0, 60px))!important;
    grid-gap: 10px;
    margin: auto
}

.mines__control {
    min-width: 100%;
    display: flex;
    justify-content: space-between;
}

.mines__input > input {
    min-width: 100%;
    margin-bottom: 15px
}

.mine-tile {
    background: #eee;
    border: 1px solid #eee;
    width: 60px;
    height: 60px;
    cursor: pointer;
}

.mine-tile.loading {
    display: flex;
    align-items: center
}

.mine-tile:hover {
    background: #fff
}

.mine-tile:not(.active).correct, .mine-tile:not(.active).wrong {
    opacity: 0.4;
}

.mine-tile.correct {
    background: url(/assets/img/win.png);
    margin-left: auto;
    margin-right: auto;
   
    transition: 0s;
    pointer-events: none
}

.mine-tile.wrong {
    background: url(/assets/img/lose.png);
    margin-left: auto;
    margin-right: auto;
    
    transition: 0s;
    pointer-events: none;
   
}

@media (max-width: 62em) {
    .mines {
        padding: 12px 12px 23px;
        justify-content: space-around;
    }
    .col-md-7 {
        order: 1;
    }
    .col-md-5 {
        order: 2;
        margin-top: 20px
    }
    .multiplayers__wrap::-webkit-scrollbar-thumb {
        background: none!important;
    }
    .mines__control {
        max-width: 170px;
    }
}

@media (min-width: 62em) {
    .col-md-5, .col-md-7 {
        width: 100%;
    }
    .col-md-5 {
        padding-right: 14px!important;
        padding-left: 32px!important;
    }

    .col-md-7 {
        padding-right: 0!important;
        padding-left: 58px!important;
    }
}

.btn__create {
    margin: 18px 0 18px 0
}

.inputs-wrap {
    display: inline-block;
    margin: auto 0;
    min-width: 100%;
    margin-top: 10px;
}

.inputs {
    display: flex;
    margin: 0 -10px;
}

.input {
    display: inline-block;
    flex: 1;
    margin: 0 10px;
}

.input-title {
    margin-bottom: 10px;
}

.form-control {
    text-align: center;
    height: 48px;
    font-size: 24px;
    /* min-width: 100%; */
    max-height: 40px;
    height: 40px!important;
    border-radius: 0px!important
}

.w-48 {
    min-width: 48%!important;
    max-width: 48%!important;
}

.wh-14 {
    width: 14px!important;
    height: 14px!important;
}

.container {
    width: auto!important;
}

.row {
    margin-right: 0!important;
    margin-left: 0!important;
}

input {
    background: #eeecfd;
    padding: 18px 21px;
    border: 1px solid #dcd9f7;
    -webkit-box-sizing: border-box;
    box-sizing: border-box;
    -webkit-box-shadow: 0 5px 35px rgb(215 211 236 / 25%);
    box-shadow: 0 5px 35px rgb(215 211 236 / 25%);
    border-radius: 4px;
    font-size: 13px;
    line-height: 15px;
    max-height: 20px;
    max-width: 170px;
    padding-right: 30px;
}

.multiplayers__wrap {
    overflow-x: scroll;
    overflow-y: hidden;
    padding-bottom: 15px;
    width: 336px;
    margin: 10px auto 0;
}

.multiplayers__wrap::-webkit-scrollbar {
    width: 5px;
    height: 8px;
    background: 0 0;
}

.multiplayers__wrap::-webkit-scrollbar-thumb {
    background: rgba(0, 0, 0, 0.1);
}

.multiplayers__wrap, .multiplier-item {
    display: flex;
}

.multiplier {
    border: 1px solid #e5e5e5;
    border-radius: 0.25em;
    padding: 0.5rem;
    display: inline-block;
    width: 82px;
    text-align: center;
}

.multiplier.active {
    background: #eee
}

.multiplier.inProcess {
    border: 1px solid #2c80ff!important;
}

.multiplier-item:last-child > .pointer {
    display: none;
}

.pointer {
    display: inline-block;
    margin: auto 8px;
}

.spinner-border {
    display: inline-block;
    width: 2rem;
    height: 2rem;
    vertical-align: text-bottom;
    border: .25em solid currentColor;
    border-right-color: currentcolor;
    border-right-color: transparent;
    border-radius: 50%;
    animation: spinner-border .75s linear infinite;
    margin: auto;
}

@keyframes spinner-border {
    to {
        transform: rotate(360deg)
    }
}
</style>